package aggSim.analyzer;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;

import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import aggSim.Grammar;
import aggSim.NetMorphism;

public class Analyzer {

	private Analyzer() {
	}

	// FIXME arc weights are not considered in analyzeTokenFiring(ObjectNet)
	static public List<Conflict> analyzeTokenFiring(final ObjectNet objectNet) {
		final Vector<Conflict> result = new Vector<Conflict>();
		for (final ONNode node : objectNet.getNodes()) {
			if (node instanceof ONPlace) {
				final EList<ONArc> outgoing = ((ONPlace) node).getOutgoing();
				if (outgoing.size() > 1) {
					for (int i = 0; i < outgoing.size() - 1; i++) {
						for (int j = i + 1; j < outgoing.size(); j++) {
							final Conflict conflict = new Conflict(
									Conflict.TRANSITION_TRANSITION_CONFLICT,
									outgoing.get(i).getTarget(), outgoing
											.get(j).getTarget());
							result.add(conflict);
						}
					}
				}
			}
		}
		return result;
	}

	static public List<Conflict> analyzeFiringTransformation(final Rule rule,
			final ObjectNet objectNet) {
		final Vector<Conflict> result = new Vector<Conflict>();
		final Vector<ONTransition> deletedTransitions = new Vector<ONTransition>();
		final Vector<ONTransition> mappedTransitions = new Vector<ONTransition>();

		for (final ONNode node : rule.getLhs().getNodes()) {
			if (node instanceof ONTransition) {
				deletedTransitions.add((ONTransition) node);
			}
		}

		for (final Mapping mapping : rule.getRhs().getMappings()) {
			if (mapping.getSource() instanceof ONTransition) {
				mappedTransitions.add((ONTransition) mapping.getSource());
			}
		}

		deletedTransitions.removeAll(mappedTransitions);

		final Grammar grammar = new Grammar();

		if (!deletedTransitions.isEmpty()) {
			final ArrayList<NetMorphism> morphisms = grammar.getMatches(rule,
					objectNet);
			for (final NetMorphism morphism : morphisms) {
				for (final ONTransition ruleTransition : deletedTransitions) {
					for (final Mapping mapping : morphism) {
						if (mapping.getSource() == ruleTransition) {
							final Conflict conflict = new Conflict(
									Conflict.RULE_TRANSITION_CONFLICT,
									morphism, mapping.getTarget());
							result.add(conflict);
						}
					}
				}
			}
		}
		return result;
	}

	static public List<Conflict> analyzeNetTransformation(
			final ObjectNet objectNet, final Rule rule1, final Rule rule2) {
		final Grammar grammar = new Grammar();
		final ArrayList<Conflict> result = new ArrayList<Conflict>();
		final ArrayList<ArrayList<NetMorphism>> criticalPairs = grammar
				.performCriticalPairAnalysis(objectNet, rule1, rule2);
		for (final ArrayList<NetMorphism> criticalPair : criticalPairs) {
			final Conflict conflict = new Conflict(Conflict.RULE_RULE_CONFLICT,
					criticalPair.get(0), criticalPair.get(1));
			result.add(conflict);
		}
		return result;
	}

}
