package aggSim.analyzer.dialogs;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Group;

import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.model.RuleUtil;
import ronmodel.Mapping;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import aggSim.analyzer.Conflict;

/**
 * @author Tony Modica
 * 
 */
public class RuleRuleConflictDialog extends AbstractConflictDialog {

	private final ObjectNet objectNet;
	private final Rule rule1;
	private final Rule rule2;

	private SashForm topSashForm;
	private SashForm rulesSashForm;
	private SashForm rule1SashForm;
	private SashForm rule2SashForm;

	private GraphicalViewer viewerON;

	private final Color rule1Color = SWTResourceManager.getColor(230, 255, 230);
	private final Color rule2Color = SWTResourceManager.getColor(230, 230, 255);

	public RuleRuleConflictDialog(final ObjectNet objectNet, final Rule rule1,
			final Rule rule2, final List<Conflict> conflicts) {
		super(conflicts);
		this.objectNet = objectNet;
		this.rule1 = rule1;
		this.rule2 = rule2;
	}

	@Override
	protected void createControl() {
		getDialogShell().setText(
				"Found " + getConflicts().size() + " Rule/Rule conflicts");
		{
			topSashForm = new SashForm(getDialogShell(), SWT.VERTICAL);
			final GridData topSashFormLData = new GridData();
			topSashFormLData.grabExcessHorizontalSpace = true;
			topSashFormLData.grabExcessVerticalSpace = true;
			topSashFormLData.horizontalAlignment = GridData.FILL;
			topSashFormLData.verticalAlignment = GridData.FILL;
			topSashForm.setLayoutData(topSashFormLData);

			rulesSashForm = new SashForm(topSashForm, SWT.HORIZONTAL);

			final Group rule1Group = new Group(rulesSashForm, SWT.NONE);
			final FillLayout groupLayout = new FillLayout(SWT.HORIZONTAL);
			rule1Group.setLayout(groupLayout);
			rule1Group.setText("Rule '" + rule1.getName() + "'");
			rule1SashForm = new SashForm(rule1Group, SWT.HORIZONTAL);

			final Group rule2Group = new Group(rulesSashForm, SWT.NONE);
			rule2Group.setLayout(groupLayout);
			rule2Group.setText("Rule '" + rule2.getName() + "'");
			rule2SashForm = new SashForm(rule2Group, SWT.HORIZONTAL);
		}
	}

	@Override
	protected void init() {
		viewerON = createGraphicalViewer(topSashForm, objectNet);
		// create viewer for lhs1
		createGraphicalViewer(rule1SashForm, rule1.getLhs(), rule1Color);
		// create viewer for rhs1
		createGraphicalViewer(rule1SashForm, rule1.getRhs(), rule1Color);
		// create viewer for lhs2
		createGraphicalViewer(rule2SashForm, rule2.getLhs(), rule2Color);
		// create viewer for rhs2
		createGraphicalViewer(rule2SashForm, rule2.getRhs(), rule2Color);
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void applyConflict(final int i) {
		// delete current mappings into objectNet
		for (final Mapping mapping : new ArrayList<Mapping>(
				objectNet.getMappings())) {
			mapping.setCodomain(null);
		}

		// set all nodes to default color
		RuleUtil.resetColors(objectNet);

		// handle the special case that all conflicts have been filtered out
		if (i == -1) {
			return;
		}

		// continue to set all nodes to default color
		RuleUtil.resetColors(rule1);
		RuleUtil.resetColors(rule2);

		// set mapping 1 of new current conflict
		for (final Mapping mapping : (List<Mapping>) getConflicts().get(i)
				.getFirst()) {
			mapping.setCodomain(objectNet);
		}

		// set mapping 2 of new current conflict
		for (final Mapping mapping : (List<Mapping>) getConflicts().get(i)
				.getSecond()) {
			mapping.setCodomain(objectNet);
		}

		// set colors for all nodes
		RuleUtil.updateColors(objectNet);
		RuleUtil.updateColors(rule1);
		RuleUtil.updateColors(rule2);
	}

	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	@Override
	protected void optionalDispose() {
		// delete current mapping into objectNet and reset colors to reset
		// everything if the objectnet and rule is used in a nother dialog to
		// show other conflicts
		for (final Mapping mapping : new ArrayList<Mapping>(
				objectNet.getMappings())) {
			mapping.setCodomain(null);
		}

		// set all nodes to default color
		RuleUtil.resetColors(rule1);
		RuleUtil.resetColors(rule2);
		RuleUtil.resetColors(objectNet);
	}

	@Override
	protected GraphicalViewer getMainViewer() {
		return viewerON;
	}

}
