package aggSim.analyzer.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Group;

import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.model.RuleUtil;
import ronmodel.Mapping;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import aggSim.analyzer.Conflict;

/**
 * @author Tony Modica
 * 
 */
public class RuleTransConflictDialog extends AbstractConflictDialog {

	private final Border highLightBorder = new LineBorder(ColorConstants.red, 3);

	private final ObjectNet objectNet;

	private final Rule rule;

	private ONTransition currentHighlighted;

	private GraphicalViewer viewerON;

	// visual stuff
	private SashForm topSashForm;
	private SashForm ruleSashForm;

	public RuleTransConflictDialog(final ObjectNet objectNet, final Rule rule,
			final List<Conflict> conflicts) {
		/**
		 * First element is mapping, second is object net transition
		 */
		super(conflicts);
		this.objectNet = objectNet;
		this.rule = rule;
	}

	@Override
	protected void createControl() {
		getDialogShell()
				.setText(
						"Found " + getConflicts().size()
								+ " Rule/Transition conflicts");

		// START >> topSashForm
		topSashForm = new SashForm(getDialogShell(), SWT.VERTICAL);
		final GridData sashFormLData = new GridData();
		sashFormLData.verticalAlignment = GridData.FILL;
		sashFormLData.horizontalAlignment = GridData.FILL;
		sashFormLData.grabExcessHorizontalSpace = true;
		sashFormLData.grabExcessVerticalSpace = true;
		topSashForm.setLayoutData(sashFormLData);
		// END << topSashForm

		final Group ruleGroup = new Group(topSashForm, SWT.NONE);
		final FillLayout groupLayout = new FillLayout(SWT.HORIZONTAL);
		ruleGroup.setLayout(groupLayout);
		ruleGroup.setText("Rule '" + rule.getName() + "'");

		ruleSashForm = new SashForm(ruleGroup, SWT.HORIZONTAL);
	}

	@Override
	protected void init() {
		viewerON = createGraphicalViewer(topSashForm, objectNet);
		// create viewer for lhs
		createGraphicalViewer(ruleSashForm, rule.getLhs());
		// create viewer for rhs
		createGraphicalViewer(ruleSashForm, rule.getRhs());
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void applyConflict(final int i) {
		// delete current mapping into objectNet
		for (final Mapping mapping : new ArrayList<Mapping>(
				objectNet.getMappings())) {
			mapping.setCodomain(null);
		}

		// "lowlight" the previously highlighted transition by removing its
		// border
		if (currentHighlighted != null) {

			final IFigure transitionFigure = ((GraphicalEditPart) viewerON
					.getEditPartRegistry().get(currentHighlighted)).getFigure();
			transitionFigure.setBorder(null);
		}

		// set all nodes to default color
		RuleUtil.resetColors(objectNet);

		// handle the special case that all conflicts have been filtered out
		if (i == -1) {
			return;
		}

		// continue to set all nodes to default color
		RuleUtil.resetColors(rule);

		// set mapping of new current conflict
		for (final Mapping mapping : (List<Mapping>) getConflicts().get(i)
				.getFirst()) {
			mapping.setCodomain(objectNet);
		}

		// set colors for all nodes
		RuleUtil.updateColors(objectNet);
		RuleUtil.updateColors(rule);

		// highlight transition figure according to conflict
		currentHighlighted = (ONTransition) getConflicts().get(i).getSecond();
		final IFigure transitionFigure = ((GraphicalEditPart) viewerON
				.getEditPartRegistry().get(currentHighlighted)).getFigure();
		transitionFigure.setBorder(highLightBorder);
	}

	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	@Override
	protected void optionalDispose() {
		// delete current mapping into objectNet and reset colors to reset
		// everything if the objectnet and rule is used in a nother dialog to
		// show other conflicts
		for (final Mapping mapping : new ArrayList<Mapping>(
				objectNet.getMappings())) {
			mapping.setCodomain(null);
		}

		RuleUtil.resetColors(rule);
		RuleUtil.resetColors(objectNet);
	}

	@Override
	protected GraphicalViewer getMainViewer() {
		return viewerON;
	}
}
