package aggSim.analyzer.dialogs;

import java.util.List;

import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;

import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import aggSim.analyzer.Conflict;

/**
 * @author Tony Modica
 * 
 */
public class TransTransConflictDialog extends AbstractConflictDialog {

	private final Border highLightBorder1 = new LineBorder(ColorConstants.red,
			3);
	private final Border highLightBorder2 = new LineBorder(ColorConstants.blue,
			3);

	private final ObjectNet objectNet;

	private ONTransition currentHighlighted1, currentHighlighted2;

	private GraphicalViewer viewerON;

	// visual stuff
	private SashForm sashForm;

	public TransTransConflictDialog(final ObjectNet objectNet,
			final List<Conflict> conflicts) {
		/**
		 * First element is mapping, second is object net transition
		 */
		super(conflicts);
		this.objectNet = objectNet;
	}

	@Override
	protected void createControl() {
		getDialogShell().setText(
				"Found " + getConflicts().size()
						+ " Transition/Transition conflicts");

		// START >> sashForm
		sashForm = new SashForm(getDialogShell(), SWT.BORDER);
		final GridData sashFormLData = new GridData();
		sashFormLData.verticalAlignment = GridData.FILL;
		sashFormLData.horizontalAlignment = GridData.FILL;
		sashFormLData.grabExcessHorizontalSpace = true;
		sashFormLData.grabExcessVerticalSpace = true;
		sashForm.setLayoutData(sashFormLData);
		// END << sashForm
	}

	@Override
	protected void init() {
		viewerON = createGraphicalViewer(sashForm, objectNet);
	}

	@Override
	protected GraphicalViewer getMainViewer() {
		return viewerON;
	}

	@Override
	protected void applyConflict(final int i) {
		// highlight transition figure according to conflict
		if (currentHighlighted1 != null) {
			// "lowlight" the previously highlighted transition by removing its
			// border
			IFigure transitionFigure = ((GraphicalEditPart) viewerON
					.getEditPartRegistry().get(currentHighlighted1))
					.getFigure();
			transitionFigure.setBorder(null);
			transitionFigure = ((GraphicalEditPart) viewerON
					.getEditPartRegistry().get(currentHighlighted2))
					.getFigure();
			transitionFigure.setBorder(null);
		}

		// handle the special case that all conflicts have been filtered out
		if (i == -1) {
			return;
		}

		currentHighlighted1 = (ONTransition) getConflicts().get(i).getFirst();
		IFigure transitionFigure = ((GraphicalEditPart) viewerON
				.getEditPartRegistry().get(currentHighlighted1)).getFigure();
		transitionFigure.setBorder(highLightBorder1);

		currentHighlighted2 = (ONTransition) getConflicts().get(i).getSecond();
		transitionFigure = ((GraphicalEditPart) viewerON.getEditPartRegistry()
				.get(currentHighlighted2)).getFigure();
		transitionFigure.setBorder(highLightBorder2);
	}

	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	@Override
	protected void optionalDispose() {
	}

}
