package muvitorkit.animation.demo;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.animation.AnimatingCommand;
import muvitorkit.animation.AnimationPathModifier;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import ronmodel.Node;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * @author Tony Modica
 * 
 */
public class AnimatingDemoAction2 extends SelectionAction {

	static public final String ID = "AnimatingDemoAction2";

	private EObject selectedModel;

	public AnimatingDemoAction2(final IWorkbenchPart part) {
		super(part);
		setDescription(ID);
		setText(ID);
		setId(ID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		final List<?> selectedObjects = getSelectedObjects();
		if (selectedObjects.size() != 1) {
			return false;
		}
		final Object selectedObject = selectedObjects.get(0);
		if (!(selectedObject instanceof EditPart)) {
			return false;
		}
		final Object selectedEditPartModel = ((EditPart) selectedObject)
				.getModel();
		if (!(selectedEditPartModel instanceof EObject)) {
			return false;
		}
		selectedModel = (EObject) selectedEditPartModel;
		return true;
	}

	@Override
	public void run() {
		// AnimatingCommand.setDebug(true);
		execute(new AnimatingDemoCommand2(selectedModel));
		// AnimatingCommand.setDebug(false);
	}

	private static class AnimatingDemoCommand2 extends AnimatingCommand {

		private final ArrayList<Node> childNodes;

		public AnimatingDemoCommand2(final EObject model) {
			// initialize animated elements
			// initializeAnimatedElement(model, model.eContainer());

			/*
			 * angepasst an RON model, da das RON das oberste angezeigte Modell
			 * im Viewer ist. Es kann auch das RON ausgewhlt und somit die
			 * HL-Nodes animiert werden.
			 */
			final boolean modelisObjectNet = model instanceof ObjectNet;
			if (modelisObjectNet) {
				initializeAnimatedElement(model, model.eContainer()
						.eContainer());
			}

			// angepasst an RON model: Arcs rausfiltern!
			childNodes = new ArrayList<Node>();

			int pathKindCounter = 0;
			AnimationPathModifier pm = null;
			for (final EObject elem : model.eContents()) {
				if (elem instanceof Node) {
					childNodes.add((Node) elem);
					switch (pathKindCounter++ % 3) {
					case 0:
						pm = AnimationPathModifier.getRandomModifier(40);
						break;
					case 1:
						pm = AnimationPathModifier.getSineModifier(40, 4);
						break;
					case 2:
						pm = AnimationPathModifier.getStandardModifier();
						break;
					}
					initializeAnimatedElement(elem, model, pm);
				}
			}

			// define original positions as initial positions
			if (modelisObjectNet) {
				specifyStep(model, ((Token) model).getPlace());
			}
			for (final EObject child : childNodes) {
				specifyStep(child, child);
			}
			nextStep(2);
			// move all nodes to corner and grow the object net token
			for (final EObject child : childNodes) {
				specifyStep(child, new Point(10, 10));
			}
			if (modelisObjectNet) {
				specifyStep(model, ((Token) model).getPlace(), 4);
			}
			// move a bit slower
			nextStep(3);
			// move all nodes to random places and shrink the object net token
			// to original size
			for (final EObject child : childNodes) {
				specifyStep(child, new Point(Math.random() * 800,
						Math.random() * 400));
				// specifyStep(child, new Point(500, 700));
			}
			if (modelisObjectNet) {
				specifyStep(model, ((Token) model).getPlace());
			}
		}

		@Override
		public void execute() {
			super.execute();
			// transfer the new positions generated by the muvitorkit.animation
			// to model
			for (final Node node : childNodes) {
				final Point newLocation = getFinalLocation(node);
				if (newLocation != null) {
					node.setX(newLocation.x);
					node.setY(newLocation.y);
				}
			}
		}

		@Override
		public void undo() {
			super.undo();
			// restore the old positions before the animation to model
			for (final Node node : childNodes) {
				final Point newLocation = getInitialLocation(node);
				if (newLocation != null) {
					node.setX(newLocation.x);
					node.setY(newLocation.y);
				}
			}
		}

	}

}
