/**
 *
 */
package muvitorkit.animation.demo;

import java.util.List;

import muvitorkit.animation.AnimatingCommand;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

/**
 * Test for size manipulation.
 * 
 * @author Tony Modica
 * 
 */
public class AnimatingDemoAction3 extends SelectionAction {

	static public final String ID = "AnimatingDemoAction3";

	private EObject selectedModel;

	public AnimatingDemoAction3(final IWorkbenchPart part) {
		super(part);
		setDescription(ID);
		setText(ID);
		setId(ID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		final List<?> selectedObjects = getSelectedObjects();
		if (selectedObjects.size() != 1) {
			return false;
		}
		final Object selectedObject = selectedObjects.get(0);
		if (!(selectedObject instanceof EditPart)
				|| selectedObject instanceof ConnectionEditPart) {
			return false;
		}
		final EditPart selectedEditPart = (EditPart) selectedObject;

		// must not select contents but some direct sub-editpart
		if (selectedEditPart.getViewer().getContents() != selectedEditPart
				.getParent()) {
			return false;
		}

		final Object selectedEditPartModel = selectedEditPart.getModel();
		if (!(selectedEditPartModel instanceof EObject)) {
			return false;
		}

		selectedModel = (EObject) selectedEditPartModel;
		return true;
	}

	@Override
	public void run() {
		// AnimatingCommand.setDebug(true);
		execute(new AnimatingDemoCommand3(selectedModel));
		// AnimatingCommand.setDebug(false);
	}

	private static class AnimatingDemoCommand3 extends AnimatingCommand {

		public AnimatingDemoCommand3(final EObject model) {
			flushAfter = true;
			setCompleteDuration(2000);
			initializeAnimatedElement(model, model.eContainer());
			specifyStep(model, model, 1);
			nextStep();
			specifyStep(model, model, 2);
			nextStep();
			specifyStep(model, model, 4);
			nextStep();
			specifyStep(model, model, 1.5);
			nextStep();
			specifyStep(model, model, 1);
		}
	}

}
