package roneditor.gef.actions.objectnet;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.actions.RONEditorActionBarContributor;
import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.gef.editparts.objectnet.ONTransitionEditPart;

/**
 * Action for firing a PetriNet's Transition.
 */
public class FireONTransitionAction extends SelectionAction {

	public static final String ID = RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_ID;

	public FireONTransitionAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_LABEL);
		setDescription(RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_DESC);
		setToolTipText(RONEditorActionBarContributor.FIRE_TRANSITION_ACTION_DESC);
	}

	/**
	 * Gets a Command and executes it.
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		final StructuredSelection selection = (StructuredSelection) getSelection();
		final ONTransitionEditPart part = (ONTransitionEditPart) selection
				.getFirstElement();
		final Command command = new FireTransitionCommand(part.getCastedModel());
		execute(command);
	}

	/**
	 * Check if one and only one {@link ONTransitionEditPart} is selected and
	 * verify it can fire.
	 * 
	 * @return true if we have selected a transition an if it can fire, false
	 *         otherwise.
	 * 
	 * @see ONTransitionEditPart#canFire()
	 */
	@Override
	protected boolean calculateEnabled() {
		final StructuredSelection selection = (StructuredSelection) getSelection();

		if (selection == null || selection.size() != 1) {
			return false;
		}

		final Object selectionElement = selection.getFirstElement();
		if (!(selectionElement instanceof ONTransitionEditPart)) {
			return false;
		}

		final ONTransitionEditPart transitionPart = (ONTransitionEditPart) selectionElement;
		return transitionPart.canFire();
	}

}
