package roneditor.gef.actions.rule;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.ui.IEditorPart;

import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.ui.rule.RulePage;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.RHS;
import ronmodel.RonmodelFactory;

/**
 * Copies the object net in the {@link RHS} to the active {@link NAC} given a
 * {@link RulePage}.
 */
public class CopyRHSToNACAction extends ObjectnetCopyAction {

	static public final String ID = "Copy RHS -> NAC";

	static private final String DESC = "Copy RHS -> NAC";

	static private final String TOOLTIP = "Copies the object net in the RHS to the active NAC";

	private final List<Mapping> mappings;

	public CopyRHSToNACAction(final RulePage rulePage, final IEditorPart editor) {
		super(editor, rulePage);
		setId(ID);
		super.setText(DESC);
		super.setDescription(DESC);
		super.setToolTipText(TOOLTIP);
		source = getRulePage().getRule().getRhs();
		mappings = getRulePage().getRule().getRhs().getMappings();
	}

	@Override
	public void run() {
		final List<EObject> copyCollection = (List<EObject>) EcoreUtil
				.copyAll(source.eContents());
		final CompoundCommand command = new CompoundCommand();

		command.add(new CreateFromCollectionCommand(copyCollection, target));

		for (final Object mappingObject : mappings) {
			final Mapping mapping = (Mapping) mappingObject;
			final ONNode lhsNode = mapping.getSource();
			final ONNode rhsNode = mapping.getTarget();
			final int sourcePosition = source.eContents().indexOf(rhsNode);
			final ONNode targetNode = (ONNode) copyCollection
					.get(sourcePosition);

			final Mapping newMapping = RonmodelFactory.eINSTANCE
					.createMapping();
			final UpdateMappingCommand updateMappingCommand = new UpdateMappingCommand();
			updateMappingCommand.setSource(lhsNode);
			updateMappingCommand.setTarget(targetNode);
			updateMappingCommand.setCodomain(target);
			updateMappingCommand.setMapping(newMapping);
			updateMappingCommand.update();

			command.add(updateMappingCommand);
		}
		execute(command);
	}

	@Override
	public void update() {
		target = getRulePage().getActiveNAC();
		super.update();
	}
}
