package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Command for connecting an {@link ONTransition} with an {@link ONPlace}.
 */
public class ConnectONNodesCommand extends Command {

	private static final String LABEL = "Connect '";

	private final ONNode source;

	private ONNode target;

	private final ONArc arc;

	/**
	 * Constructor.
	 */
	public ConnectONNodesCommand(final ONNode source, final ONArc arc) {
		this.source = source;
		this.arc = arc;
	}

	public boolean isSourcePlace() {
		return source instanceof ONPlace;
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	@Override
	public boolean canExecute() {
		if (source == null || target == null || arc == null || source == target) {
			return false;
		}
		return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	@Override
	public void execute() {
		final ObjectNet objectnet = source.getObjectNet();
		arc.setName("");
		arc.setObjectNet(objectnet);
		arc.setSource(source);
		arc.setTarget(target);
	}

	/**
	 * Redo command execution.
	 */
	@Override
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		arc.setObjectNet(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Set the given {@link ONNode} as target.
	 * 
	 * @param target
	 */
	public void setTarget(final ONNode target) {
		this.target = target;
		setLabel(LABEL + source.getName() + "' -> '" + target.getName() + "'");
	}

	public ONArc getArc() {
		return arc;
	}

	/**
	 * @return the source
	 */
	public ONNode getSource() {
		return source;
	}
}
