package roneditor.gef.commands.objectnet;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * Command for deleting a {@link ONArc} between two {@link ONNode}s.
 */
public class DeleteArcCommand extends Command {

	private static final String LABEL = "Delete ONArc '";

	private final ONArc arc;

	private ONNode source;

	private ONNode target;

	private ObjectNet objectNet;

	public DeleteArcCommand(final ONArc arc) {
		this.arc = arc;
	}

	@Override
	public boolean canExecute() {
		return arc.getSource() != null && arc.getTarget() != null;
	}

	/**
	 * Execute the command.
	 */
	@Override
	public void execute() {
		/*
		 * FIXED when this Command is used in a DeleteNodeCommand while deleting
		 * several ONNodes, this arc may have been already deleted. In this case
		 * this command should do nothing! (the compound command verifies
		 * execution before deleting any arc)
		 */
		if (!canExecute()) {
			return;
		}
		setLabel(LABEL + arc.getSource().getName() + "' -> '"
				+ arc.getTarget().getName() + "'");
		source = arc.getSource();
		target = arc.getTarget();
		objectNet = arc.getObjectNet();
		arc.setObjectNet(null);
		arc.setSource(null);
		arc.setTarget(null);
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		/*
		 * FIXED see above, if objectnet is null the command has not been
		 * executed before
		 */
		if (objectNet == null) {
			return;
		}
		arc.setObjectNet(objectNet);
		arc.setSource(source);
		arc.setTarget(target);
	}
}
