package roneditor.gef.commands.objectnet;

import java.util.List;

import muvitorkit.animation.AnimatingCommand;
import muvitorkit.commands.SimpleDeleteEObjectCommand;

import org.eclipse.gef.commands.CompoundCommand;

import roneditor.gef.commands.rule.DeleteMappingCommand;
import roneditor.model.RuleUtil;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.Rule;

/**
 * The <code>DeleteONNodeCompoundCommand</code> is used to delete an
 * {@link ONNode} and all the connected {@link ONArc}. Additionally, if the
 * {@link ONNode} is part of an {@link Rule}, all associated {@link Mapping}
 * will be deleted.
 */
public class DeleteNodeCommand extends CompoundCommand {
	private static final String LABEL = "Delete ONNode ";

	public DeleteNodeCommand(final ONNode node) {
		super(LABEL + node.getName());

		// Delete Mappings
		final ObjectNet objectNet = node.getObjectNet();
		if (objectNet instanceof LHS || objectNet instanceof RHS
				|| objectNet instanceof NAC) {
			final List<Mapping> mappings = objectNet instanceof LHS ? RuleUtil
					.getAllCompleteMappings(((LHS) objectNet).getRule())
					: objectNet.getMappings();
			for (final Mapping mapping : mappings) {
				if (mapping.getTarget() == node || mapping.getSource() == node) {
					add(new DeleteMappingCommand(mapping));
				}
			}
		}

		// Delete ONArcs
		for (final ONArc link : node.getIncoming()) {
			add(new DeleteArcCommand(link));
		}
		for (final ONArc link : node.getOutgoing()) {
			add(new DeleteArcCommand(link));
		}

		final AnimatingCommand aniComm = new AnimatingCommand();
		aniComm.setCompleteDuration(350);
		aniComm.flushBefore = true;
		aniComm.initializeAnimatedElement(node, node.getObjectNet());
		aniComm.specifyStep(node, node, 1);
		aniComm.nextStep();
		aniComm.specifyStep(node, node, 1.5);
		aniComm.nextStep(2);
		aniComm.specifyStep(node, node, 0);
		add(aniComm);

		// Delete ONNode itself
		add(new SimpleDeleteEObjectCommand(node));
	}

}
