package roneditor.gef.commands.ron;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;

import ronmodel.HLPlace;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * helper class that copies one {@link ObjectNet} from a {@link HLPlace} to
 * <i>one</i> new {@link ObjectNet} on the target {@link HLPlace}.
 */
public class CopyTokenToHLPlaceCommand extends Command {
	private final HLPlace target;

	private Token token;

	private HLPlace oldPlace;

	private int oldIndex;

	private final boolean isMove;

	public CopyTokenToHLPlaceCommand(final Token token, final HLPlace target,
			final boolean isMove) {
		this.target = target;
		this.token = token;
		this.isMove = isMove;

	}

	@Override
	public void execute() {
		if (isMove) {
			// store originating place and position
			oldPlace = token.getPlace();
			oldIndex = oldPlace.getTokens().indexOf(token);
		} else {
			// only copy if explicitly demanded
			final String name = token.getName();
			token = EcoreUtil.copy(token);
			token.setName(name);
			// there is no originating place, token has just been created
			// (possibly union)
			oldPlace = null;
		}
		token.setPlace(target);
	}

	@Override
	public void undo() {
		if (isMove) {
			if (target == oldPlace) {
				// if token has been moved to the same place just restore old
				// position
				oldPlace.getTokens().move(oldIndex, token);
			} else {
				oldPlace.getTokens().add(oldIndex, token);
			}
		} else {
			// token has no originating place, just delete
			token.setPlace(null);
		}
	}

	@Override
	public void redo() {
		// everything has been prepared in execute(), just set the place again
		token.setPlace(target);
	}

}
