package roneditor.gef.commands.ron;

import org.eclipse.gef.commands.Command;

import ronmodel.HLNode;
import ronmodel.RON;

/**
 * Command for creating a generic node
 * 
 */
public class CreateHLNodeCommand extends Command {

	private static final String LABEL = "Create HLNode ";

	final private HLNode hlNode;

	private int x;

	private int y;

	final private RON ron;

	private String name;

	public CreateHLNodeCommand(final RON ron, final HLNode hlNode) {
		super(LABEL);
		this.ron = ron;
		this.hlNode = hlNode;
		setName("");
	}

	/**
	 * Entry point for command execution.
	 */
	@Override
	public void execute() {
		hlNode.setName(name);
		hlNode.setRon(ron);
		hlNode.setX(x);
		hlNode.setY(y);
	}

	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	@Override
	public boolean canExecute() {
		return name != null;
	}

	/**
	 * Redo command execution.
	 */
	@Override
	public void redo() {
		hlNode.setRon(ron);
	}

	/**
	 * Undo command execution.
	 */
	@Override
	public void undo() {
		hlNode.setRon(null);
	}

	public void setLocation(final int x, final int y) {
		this.x = x;
		this.y = y;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(final String name) {
		this.name = name;
		setLabel(LABEL + "'" + name + "'");
	}

	public HLNode getHLNode() {
		return hlNode;
	}

}
