package roneditor.gef.commands.ron;

import muvitorkit.animation.AnimatingCommand;
import muvitorkit.commands.SimpleDeleteEObjectCommand;

import org.eclipse.gef.commands.CompoundCommand;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.RON;

/**
 * Command for removing a {@link HLNode} from its parent {@link RON}. This is
 * undoable.
 */
public class DeleteHLNodeCommand extends CompoundCommand {

	private static final String LABEL = "Delete HLNode ";

	public DeleteHLNodeCommand(final HLNode node) {
		super(LABEL + node.getName());

		// Delete ONArcs
		for (final HLArc link : node.getIncoming()) {
			add(new DeleteHLArcCommand(link));
		}
		for (final HLArc link : node.getOutgoing()) {
			add(new DeleteHLArcCommand(link));
		}

		final AnimatingCommand aniComm = new AnimatingCommand();
		aniComm.setCompleteDuration(350);
		aniComm.flushBefore = true;
		aniComm.initializeAnimatedElement(node, node.getRon());
		aniComm.specifyStep(node, node, 1);
		aniComm.nextStep();
		aniComm.specifyStep(node, node, 1.5);
		aniComm.nextStep(2);
		aniComm.specifyStep(node, node, 0);
		add(aniComm);

		// Delete HLNode (a "pure" one) itself
		add(new SimpleDeleteEObjectCommand(node));
	}
}
