package roneditor.gef.commands.ron;

import java.util.List;

import muvitorkit.animation.AnimatingCommand;
import muvitorkit.ui.utils.GenericUtils;

import org.eclipse.gef.commands.CompoundCommand;

import roneditor.gef.actions.ron.RunAutoSimulationAction;
import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.model.ONTransitionUtil;
import roneditor.ui.dialogs.FireDialog;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * 
 * @author seb implements HLFireTransition :)
 */
public class HLTransitionFireCommand extends CompoundCommand {

	private static final String LABEL = "Fire Transition '";

	public HLTransitionFireCommand(final HLTransition transition,
			final boolean forceDialog) {
		final HLPlace inPlace = (HLPlace) transition.getIncoming().get(0)
				.getSource();
		final HLPlace outPlace = (HLPlace) transition.getOutgoing().get(0)
				.getTarget();

		if (inPlace.getTokens().size() > 0) {
			final ObjectNet objectNet = (ObjectNet) inPlace.getTokens().get(0);
			ONTransition ontransition = null;
			if (forceDialog) {
				final FireDialog dialog = new FireDialog(objectNet);
				dialog.open();
				if (dialog.isCanceled()) {
					return;
				}
				ontransition = dialog.getTransitionToFire();
			} else {
				final List<ONTransition> activeTransitions = ONTransitionUtil
						.getActivated(objectNet.getNodes());
				if (activeTransitions.size() == 0) {
					// No active transition, this command will be not executable
					return;
				}
				if (activeTransitions.size() == 1) {
					// Only one active transition
					ontransition = activeTransitions.get(0);
				} else if (!RunAutoSimulationAction.isSemiAuto()) {
					// semi-automatic has been turned off, select a random
					// transition without showing dialog
					ontransition = GenericUtils.getRandom(activeTransitions);
				} else {
					final FireDialog dialog = new FireDialog(objectNet);
					dialog.open();
					if (dialog.isCanceled()) {
						return;
					}
					ontransition = dialog.getTransitionToFire();
				}
			}
			/*
			 * animation: move objectnet from pre-domain place to transition,
			 * done without overwriting initialize() because at this time the
			 * element to be animated exists (but could have done so ntl.)
			 */
			AnimatingCommand aniComm = new AnimatingCommand();
			aniComm.setCompleteDuration(750);
			aniComm.initializeAnimatedElement(objectNet, transition.getRon());
			aniComm.specifyStep(objectNet, objectNet.getPlace(), 1);
			aniComm.nextStep();
			aniComm.specifyStep(objectNet, transition, 2);
			add(aniComm);

			// Fire the transition
			add(new FireTransitionCommand(ontransition));

			/*
			 * animation: move back the token from the transition to the place
			 */
			aniComm = new AnimatingCommand();
			aniComm.setCompleteDuration(750);
			aniComm.initializeAnimatedElement(objectNet, transition.getRon());
			aniComm.specifyStep(objectNet, transition, 2);
			aniComm.nextStep();
			aniComm.specifyStep(objectNet, outPlace, 1);
			aniComm.flushAfter = true;
			add(aniComm);

			add(new CopyTokenToHLPlaceCommand(objectNet, outPlace, true));

			setLabel(LABEL + ontransition.getName() + "' in net '"
					+ ontransition.getObjectNet().getName() + "' on HLTrans '"
					+ transition.getName() + "'");
		}
	}
}
