/**
 *
 */
package roneditor.gef.commands.rule;

import org.eclipse.gef.commands.Command;

import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * {@link Command} to create a {@link Mapping} between two {@link ONNode}. The
 * <code>CreateMappingCommand</code> needs a {@link ONNode} as source and
 * target, and a {@link ObjectNet} to refer to as codomain. The {@link Command}
 * when executed will modify the {@link Mapping} accordingly.
 * 
 * This {@link Command} should not be used, use {@link UpdateMappingCommand}
 * instead.
 */
public class CreateMappingCommand extends Command {

	private static final String LABEL = "Create Mapping '";

	private final Mapping mapping;

	private final ObjectNet codomain;

	private final ONNode source;

	private final ONNode target;

	public CreateMappingCommand(final Mapping mapping,
			final ObjectNet codomain, final ONNode source, final ONNode target) {
		super(LABEL + source.getName() + "' -> '" + target.getName() + "'");
		this.mapping = mapping;
		this.codomain = codomain;
		this.source = source;
		this.target = target;
	}

	@Override
	public boolean canExecute() {
		if (source == null || target == null || mapping == null
				|| source == target || codomain == null) {
			return false;
		}

		if (!(source.getObjectNet() instanceof LHS)) {
			return false;
		}

		if (source.getObjectNet() == target.getObjectNet()) {
			return false;
		}

		if (target.getClass() != source.getClass()) {
			return false;
		}

		return true;
	}

	/**
	 * Entry point for the command execution.
	 */
	@Override
	public void execute() {
		mapping.setSource(source);
		mapping.setTarget(target);
		mapping.setCodomain(codomain);
	}

	@Override
	public void undo() {
		mapping.setCodomain(null);
		mapping.setSource(null);
		mapping.setTarget(null);
	}
}
