package roneditor.gef.editparts.objectnet;

import muvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart;
import muvitorkit.gef.editparts.AdapterConnectionEditPart;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editparts.objectnet.policies.ONArcEditPolicy;
import roneditor.gef.editpolicies.ArcEndpointEditPolicy;
import roneditor.gef.figures.objectnet.ArcFigure;
import roneditor.model.properties.ONArcPropertySource;
import ronmodel.ONArc;
import ronmodel.RonmodelPackage;

/**
 * {@link EditPart} for a {@link ONArc}.
 */
public class ONArcEditPart extends AdapterConnectionEditPart<ONArc> implements
		IGraphicalDirectEditPart {

	/**
	 * Constructor
	 * 
	 * @param link
	 */
	public ONArcEditPart(final ONArc link) {
		super(link);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ArcEndpointEditPolicy());
		// Not used because we use automatic routing of connections
		// installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new
		// ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new ONArcEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		return new ArcFigure(getCastedModel().getName(), getCastedModel()
				.getWeight());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		final Connection c = getConnectionFigure();
		((ArcFigure) c).setWeight(getCastedModel().getWeight());
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterConnectionEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new ONArcPropertySource(getCastedModel());
	}

	/**
	 * Updates the ONTransition that is connected to this ONArc. This is done
	 * every time when the model is changed (despite its name).
	 */
	public void updateTransitionFiringStatus() {
		if (getSource() instanceof ONTransitionEditPart) {
			((ONTransitionEditPart) getSource()).updateFireEnabled();
		}
		if (getTarget() instanceof ONTransitionEditPart) {
			((ONTransitionEditPart) getTarget()).updateFireEnabled();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterConnectionEditPart#notifyChanged(org.
	 * eclipse.emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.ON_ARC__SOURCE:
		case RonmodelPackage.ON_ARC__TARGET:
		case RonmodelPackage.ON_ARC__WEIGHT:
			refreshVisuals();
			/*
			 * This is done every time an arc is being connected to its source
			 * or target, so that when the arc has been set completely, a newly
			 * connected transition is updated correctly
			 */
			updateTransitionFiringStatus();
			break;
		case RonmodelPackage.ON_ARC__NAME:
			refreshVisuals();
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.ON_ARC__WEIGHT;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditValidator()
	 */
	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seemuvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart#
	 * getValueLabelTextBounds()
	 */
	@Override
	public Rectangle getValueLabelTextBounds() {
		return ((ArcFigure) getFigure()).getValueLabelTextBounds();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seemuvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart#
	 * updateValueDisplay(java.lang.String)
	 */
	@Override
	public void updateValueDisplay(final String name) {
		((ArcFigure) getFigure()).updateValueDisplay(name);
	}

}
