package roneditor.gef.editparts.objectnet;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.gef.editparts.objectnet.policies.ONTransitionEditPolicy;
import roneditor.gef.figures.objectnet.TransitionFigure;
import roneditor.model.ONTransitionUtil;
import roneditor.model.RuleUtil;
import roneditor.model.properties.ONTransitionPropertySource;
import ronmodel.ONTransition;

/**
 * EditPart for an ONTransition.
 */
public class ONTransitionEditPart extends ONNodeEditPart<ONTransition> {

	/**
	 * @param transition
	 */
	public ONTransitionEditPart(final ONTransition transition) {
		super(transition);
	}

	/**
	 * Overridden to fire the transition on double-click.
	 */
	@Override
	protected void performOpen() {
		getRoot().getViewer().getEditDomain().getCommandStack()
				.execute(new FireTransitionCommand(getCastedModel()));
	}

	/**
	 * Create the edit policies for the Transition
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();

		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ONTransitionEditPolicy());
	}

	/**
	 * Update the color of the figure depending if the transition can fire. Then
	 * execute super method.
	 * 
	 * @see objectnet.ONNodeEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		// This yields true if this transition has not been assigned any color,
		// so we can manually set a color showing enablement status of firing.
		if (RuleUtil.getIncidentMappings(getCastedModel()).isEmpty()) {
			((TransitionFigure) getFigure()).setFireColor(canFire());
		}
	}

	public void updateFireEnabled() {
		refreshVisuals();
	}

	/**
	 * Check if the Transition of the model can fire.
	 * 
	 * @see ONTransitionUtil#canFire(ONTransition)
	 * 
	 * @return true if the Transition can fire
	 */
	public boolean canFire() {
		return ONTransitionUtil.canFire(getCastedModel());
	}

	/**
	 * Create the TransitionFigure as figure for the EditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		return new TransitionFigure();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new ONTransitionPropertySource(getCastedModel());
	}

}
