package roneditor.gef.editparts.objectnet;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Factory class for graphical edit parts. Here, the corresponding edit parts of
 * a network are created.
 */
public class ObjectNetEditPartFactory implements EditPartFactory {

	/**
	 * Create an instance of the requested EditPart in the given context.
	 * Register the eAdapters of the Model in the rulePage if applicable.
	 * 
	 * @param context
	 *            The context
	 * @param obj
	 *            The type from the model to create the EditPart for
	 * 
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	@Override
	public EditPart createEditPart(final EditPart context, final Object obj) {
		if (obj instanceof ObjectNet) {
			return new ObjectNetEditPart((ObjectNet) obj);
		} else if (obj instanceof ONTransition) {
			return new ONTransitionEditPart((ONTransition) obj);
		} else if (obj instanceof ONPlace) {
			return new ONPlaceEditPart((ONPlace) obj);
		} else if (obj instanceof ONArc) {
			return new ONArcEditPart((ONArc) obj);
		}

		Assert.isTrue(
				obj == null,
				"ObjectNetEditPartFactory could not create an EditPart for the model element of type "
						+ obj);

		return null;
	}
}
