package roneditor.gef.editparts.ron;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editparts.objectnet.policies.ONPlaceEditPolicy;
import roneditor.gef.editparts.ron.policies.HLPlaceEditPolicy;
import roneditor.gef.editparts.ron.policies.HLPlaceFlowLayoutEditPolicy;
import roneditor.gef.figures.ron.HLPlaceFigure;
import roneditor.model.properties.HLPlacePropertySource;
import ronmodel.HLPlace;
import ronmodel.ONPlace;
import ronmodel.RonmodelPackage;
import ronmodel.Token;

/**
 * EditPart for a HLPlaces.
 */
public class HLPlaceEditPart extends HLNodeEditPart<HLPlace> {

	/**
	 * Constructor
	 * 
	 * @param place
	 */
	public HLPlaceEditPart(final HLPlace place) {
		super(place);
	}

	/**
	 * Update the figure and refresh the display.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		// Refresh all children
		for (final Object o : getChildren()) {
			((SimpleTokenEditPart) o).refreshVisuals();
		}
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_PLACE__OUTGOING:
			refreshSourceConnections();
			break;
		case RonmodelPackage.HL_PLACE__INCOMING:
			refreshTargetConnections();
			break;
		case RonmodelPackage.HL_PLACE__TOKENS:
			refreshChildren();
			refreshVisuals();
			break;
		default:
			refreshVisuals();
			break;
		}
		super.notifyChanged(notification);
	}

	/**
	 * Create the {@link HLPlaceFigure} as figure for the {@link EditPart}
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		final HLPlaceFigure hlPlaceFigure = new HLPlaceFigure();
		hlPlaceFigure.setType(getCastedModel().getType());
		return hlPlaceFigure;
	}

	/**
	 * Delegate the content pane to the content pane of the
	 * {@link HLPlaceFigure}
	 */
	@Override
	public IFigure getContentPane() {
		final HLPlaceFigure placeFigure = (HLPlaceFigure) getFigure();
		return placeFigure.getContentPane();
	}

	/**
	 * Return the tokens on the {@link HLPlace} as it's children so the
	 * {@link EditPart} can be associated.
	 */
	@Override
	protected List<Token> getModelChildren() {
		return getCastedModel().getTokens();
	}

	/**
	 * Install the {@link ONPlace} specific edit policies (currently only the
	 * {@link ONPlaceEditPolicy}).
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new HLPlaceEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE,
				new HLPlaceFlowLayoutEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new HLPlacePropertySource(getCastedModel());
	}
}
