package roneditor.gef.editparts.ron;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.gef.editparts.AdapterGraphicalEditPart;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ShortestPathConnectionRouter;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.RONXYLayoutEditPolicy;
import roneditor.model.properties.RONPropertySource;
import ronmodel.RON;
import ronmodel.RonmodelPackage;

/**
 * EditPart for the main component {@link RON}
 */
public class RONEditPart extends AdapterGraphicalEditPart<RON> {

	private Label nameLabel;

	/**
	 * Constructor
	 * 
	 * @param ron
	 *            the {@link RON}
	 */
	public RONEditPart(final RON ron) {
		super(ron);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		final FreeformLayer layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());

		/**
		 * @author seb set a fancy ShortestPathConnectionRouter with FanRouter
		 */
		final ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
		cLayer.setAntialias(SWT.ON);
		final FanRouter frouter = new FanRouter();
		final ShortestPathConnectionRouter router = new ShortestPathConnectionRouter(
				layer);
		frouter.setSeparation(20);
		router.setSpacing(20);
		frouter.setNextRouter(router);
		cLayer.setConnectionRouter(frouter);

		return layer;
	}

	/**
	 * Install the EditPolicies for the EditPart. In particular we want to have
	 * the {@link RONEditPartGraphLayoutEditPolicy} assigned.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new RONXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.NODE_ROLE, null);
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, null);
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new RootComponentEditPolicy());
	}

	/**
	 * Return all children of the Model
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List<EObject> getModelChildren() {
		final List<EObject> ronChildren = new ArrayList<EObject>();
		ronChildren.addAll(getCastedModel().getNodes());
		return ronChildren;
	}

	/**
	 * When activated a {@link Label} showing the name of the {@link RON} is
	 * shown in the upper left corner of the {@link EditPart}
	 * 
	 * @see muvitorkit.gef.editparts.AdapterGraphicalEditPart#activate()
	 */
	@Override
	public void activate() {

		updateFigure();

		nameLabel = new Label();
		nameLabel.setFont(SWTResourceManager.getFont("Sans", 14, SWT.BOLD));
		nameLabel.setForegroundColor(Display.getCurrent().getSystemColor(
				SWT.COLOR_GRAY));
		nameLabel.setText(getCastedModel().getName());
		getFigure().add(nameLabel, new Rectangle(10, 10, -1, -1));
		super.activate();
	}

	/**
	 * Update the figure. If the EditPart is selected, the border will be
	 * thickened.
	 */
	protected void updateFigure() {
		final FreeformLayer layer = (FreeformLayer) getFigure();
		getFigure().setBackgroundColor(ColorConstants.white);
		if (getSelected() == 0) { // not selected
			layer.setBorder(new LineBorder(1));
			layer.setForegroundColor(SWTResourceManager.getColor(0, 0, 0));
		} else { // selected
			layer.setBorder(new LineBorder(2));
			layer.setForegroundColor(SWTResourceManager.getColor(150, 0, 0));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		updateFigure();
		getFigure().repaint();
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new RONPropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		if (featureId == RonmodelPackage.RON__NAME) {
			nameLabel.setText(getCastedModel().getName());
		}

		final int type = notification.getEventType();
		switch (type) {
		case Notification.ADD:
		case Notification.ADD_MANY:
		case Notification.REMOVE:
		case Notification.REMOVE_MANY:
			refreshChildren();
			break;
		case Notification.SET:
			refreshVisuals();
			break;
		}
	}

	/**
	 * Called when the state of the selection is changed. Simply refreshes the
	 * visuals to show if the EditPart is selected.
	 */
	@Override
	protected void fireSelectionChanged() {
		super.fireSelectionChanged();
		refreshVisuals();
	}
}
