package roneditor.gef.editparts.ron.policies;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.editparts.ron.HLNodeEditPart;
import roneditor.gef.figures.objectnet.NodeFigure;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.Mapping;

/**
 * {@link GraphicalNodeEditPolicy} for the {@link HLNodeEditPart}. Provides some
 * convenience functions and initiates the feedback and connection handling of
 * {@link Mapping} commands.
 */
public class HLNodeEditPolicy extends GraphicalNodeEditPolicy {

	/**
	 * Return the casted model of the host {@link EditPart}.
	 * 
	 * @return
	 */
	protected HLNode getCastedModel() {
		return (HLNode) getHost().getModel();
	}

	/**
	 * If we create a {@link Mapping} we want to have another feedback. This
	 * method is called while the connection is not finished.
	 */
	@Override
	public void showSourceFeedback(final Request request) {
		if (request instanceof CreateConnectionRequest) {
			// final CreateConnectionRequest createConnectionRequest =
			// (CreateConnectionRequest) request;
			// final Object requestingObject = createConnectionRequest
			// .getNewObject();
			// if (requestingObject instanceof Mapping) {
			((NodeFigure) getHostFigure()).setGlow(true);

			// return;
			// }
		}
		super.showSourceFeedback(request);
	}

	/**
	 * If we have create a {@link Mapping} we want to have another feedback.
	 * This method is called when the connection is finished.
	 */
	@Override
	public void eraseSourceFeedback(final Request request) {
		if (request instanceof CreateConnectionRequest) {
			final CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			final Object requestingObject = createConnectionRequest
					.getNewObject();
			if (requestingObject instanceof Mapping) {
				((NodeFigure) getHostFigure()).setGlow(false);
				return;
			}
		}
		super.eraseSourceFeedback(request);
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the
	 * completion {@link Command}. When our start {@link Command} is an
	 */
	@Override
	protected Command getConnectionCompleteCommand(
			final CreateConnectionRequest request) {
		return null;
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the creation
	 * {@link Command}. Handles the creation of the
	 */
	@Override
	protected Command getConnectionCreateCommand(
			final CreateConnectionRequest request) {
		return null;
	}

	@Override
	protected Command getReconnectSourceCommand(final ReconnectRequest request) {
		return null;
	}

	@Override
	protected Command getReconnectTargetCommand(final ReconnectRequest request) {
		return null;
	}

	/**
	 * Helper method to avoid doubly connected nodes.
	 * 
	 * @param source
	 *            A node to test if it is already connected to this EditParts
	 *            model
	 * @return <code>true</code> if source is already connected to this
	 *         EditParts model
	 */
	protected boolean isConnectedByArc(final HLNode source) {
		for (final HLArc arc : source.getOutgoing()) {
			if (arc.getTarget() == getHost().getModel()) {
				return true;
			}
		}
		return false;
	}

}
