package roneditor.gef.editparts.ron.policies;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.gef.editparts.policies.MuvitorFlowLayoutEditPolicy;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.gef.commands.objectnet.ChangeTokenOrderCommand;
import roneditor.gef.commands.ron.AddTokenCommand;
import roneditor.gef.commands.ron.MoveMultipleTokenCommand;
import roneditor.gef.editparts.ron.SimpleTokenEditPart;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import ronmodel.Token;

public class HLPlaceFlowLayoutEditPolicy extends MuvitorFlowLayoutEditPolicy {

	/**
	 * The {@link Request} should be of type {@link ChangeBoundsRequest} which
	 * contains the moved {@link EditPart}s. Hence we return a
	 * {@link MoveMultipleTokenCommand} that handles the actual model change.
	 */
	@Override
	protected Command createAddCommand(final EditPart edit, final EditPart after) {
		if (edit instanceof SimpleTokenEditPart) {
			final List<Token> modelList = new ArrayList<Token>();
			modelList.add((Token) edit.getModel());
			return new MoveMultipleTokenCommand(modelList, getCastedModel());
		}
		return null;
	}

	/**
	 * @return The model of the host {@link EditPart} casted as {@link HLPlace}.
	 */
	private HLPlace getCastedModel() {
		return (HLPlace) getHost().getModel();
	}

	/**
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	@Override
	protected Command getCreateCommand(final CreateRequest request) {
		final Object newObjectType = request.getNewObjectType();
		if (newObjectType == Token.class) {
			AddTokenCommand createCommand = null;
			Token newToken;
			final RonmodelFactory factory = RonmodelFactory.eINSTANCE;
			if (getCastedModel().getType().equals(HLPlaceType.NET_LITERAL)) {
				newToken = factory.createObjectNet();
			} else {
				final Rule newRule = factory.createRule();
				newRule.setLhs(factory.createLHS());
				newRule.setRhs(factory.createRHS());
				newRule.getLhs().setName("lhs");
				newRule.getRhs().setName("rhs");
				newToken = newRule;
			}
			createCommand = new AddTokenCommand(getCastedModel(), newToken);
			createCommand.setName("new Token");
			return createCommand;
		}
		return null;
	}

	@Override
	protected Command createMoveChildCommand(final EditPart child,
			final EditPart after) {
		final Token afterToken = after == null ? null : (Token) after
				.getModel();
		return new ChangeTokenOrderCommand((HLPlace) getHost().getModel(),
				(Token) child.getModel(), afterToken);
	}

}
