package roneditor.gef.editparts.tree;

import muvitorkit.gef.directedit.IDirectEditPart;
import muvitorkit.gef.editparts.AdapterTreeEditPart;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.RONIconUtil;
import roneditor.gef.editparts.ron.policies.HLNodeComponentEditPolicy;
import roneditor.model.properties.HLTransitionPropertySource;
import ronmodel.HLTransition;
import ronmodel.RonmodelPackage;

/**
 * TreeEditPart for ObjectNets.
 * 
 * @author Tony Modica
 */
public class HLTransitionTreeEditPart extends AdapterTreeEditPart<HLTransition>
		implements IDirectEditPart {

	/**
	 * @param model
	 */
	public HLTransitionTreeEditPart(final HLTransition model) {
		super(model);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new HLNodeComponentEditPolicy());
	}

	@Override
	protected String getText() {
		if (!getCastedModel().isSetValid() || !getCastedModel().isValid()) {
			return super.getText() + " (invalid)";
		}
		return super.getText();
	}

	@Override
	protected void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_TRANSITION__VALID:
			refreshVisuals();
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	@Override
	protected Image getImage() {
		return RONIconUtil.getIcon(RonmodelPackage.HL_TRANSITION,
				getCastedModel().getKind().getValue(), 16);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.editparts.AdapterTreeEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new HLTransitionPropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.HL_TRANSITION__NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditValidator()
	 */
	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}
}
