package roneditor.gef.editpolicies;

import muvitorkit.gef.editparts.policies.MuvitorXYLayoutEditPolicy;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.gef.commands.SetConstraintCommand;
import roneditor.gef.commands.objectnet.CreateNodeCommand;
import roneditor.gef.commands.ron.CreateHLNodeCommand;
import roneditor.gef.commands.ron.CreateHLPlaceCommand;
import roneditor.gef.commands.ron.CreateHLTransitionCommand;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.Node;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RON;

/**
 * EditPolicy for XY-Layout.
 */
public class RONXYLayoutEditPolicy extends MuvitorXYLayoutEditPolicy {

	/**
	 * Handles requests for creation of {@link Command} that generates an
	 * {@link HLPlace} or {@link HLTransition}.
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	@Override
	protected Command getCreateCommand(final CreateRequest request) {
		final Object newObjectType = request.getNewObjectType();

		if (newObjectType == HLTransition.class
				|| newObjectType == HLPlace.class) {
			CreateHLNodeCommand createCommand = null;
			if (newObjectType == HLTransition.class) {
				createCommand = new CreateHLTransitionCommand((RON) getHost()
						.getModel(), (HLTransition) request.getNewObject());
			} else {
				createCommand = new CreateHLPlaceCommand((RON) getHost()
						.getModel(), (HLPlace) request.getNewObject());
			}

			final Rectangle constraint = (Rectangle) getConstraintFor(request);
			createCommand.setLocation(constraint.x, constraint.y);
			return createCommand;
		}

		if (newObjectType == ONTransition.class
				|| newObjectType == ONPlace.class) {
			final CreateNodeCommand createCommand = new CreateNodeCommand(
					(ONNode) request.getNewObject(), (ObjectNet) getHost()
							.getModel());

			final Rectangle constraint = (Rectangle) getConstraintFor(request);
			createCommand.setLocation(constraint.x, constraint.y);
			return createCommand;
		}

		return null;
	}

	/**
	 * @return A SetConstraintCommand with the model of th host EditPart and the
	 *         constraint of its new delta set.
	 * 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	@Override
	protected Command createChangeConstraintCommand(final EditPart child,
			final Object constraint) {
		final SetConstraintCommand locationCommand = new SetConstraintCommand(
				(Node) child.getModel());
		locationCommand.setNewPosition(((Rectangle) constraint).getLocation());
		return locationCommand;
	}

	@Override
	protected void setConstraint(final EObject model, final Rectangle constraint) {
		if (model instanceof Node) {
			final Node node = (Node) model;
			node.setX(constraint.x);
			node.setY(constraint.y);
		}
	}

}
