package roneditor.gef.figures.objectnet;

import muvitorkit.gef.editparts.policies.IGhostFigureProvider;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;

/**
 * Represents the graphical layout of a place.
 */
public class TransitionFigure extends NodeFigure implements
		IGhostFigureProvider {

	public TransitionFigure() {
		super();
		// setBackgroundColor(new Color(null, 225, 255, 220));
		incomingConnectionAnchor = new ChopboxAnchor(this);
		outgoingConnectionAnchor = new ChopboxAnchor(this);
		setFireColor(false);
	}

	/**
	 * Fill the shape with a fancy gradient.
	 */
	@Override
	protected void fillShape(final Graphics graphics) {
		if (graphics.getXORMode()) {
			graphics.fillRectangle(getBounds());
		} else {
			graphics.fillGradient(getBounds(), true);
		}
	}

	/**
	 * Draw the outline of the shape. Red when glowing, black otherwise.
	 */
	@Override
	protected void outlineShape(final Graphics graphics) {
		if (glow) {
			graphics.setForegroundColor(ColorConstants.red);
		} else {
			graphics.setForegroundColor(ColorConstants.black);
		}
		super.outlineShape(graphics);
	}

	// /**
	// * @see Shape#outlineShape(Graphics)
	// */
	// private void glowShape(final Graphics graphics) {
	// final Rectangle r = getBounds();
	// for (int i = 0; i < 4; i++) {
	// final int x = r.x + i;
	// final int y = r.y + i;
	// final int w = r.width - i * 2;
	// final int h = r.height - i * 2;
	// graphics.drawRectangle(x, y, w, h);
	// }
	// }

	public void setIcon(final Image image) {
		nameLabel.setIcon(image);
	}

	@Override
	public void validate() {
		// respect possible borders
		Insets bIns = IFigure.NO_INSETS;
		if (getBorder() != null) {
			bIns = getBorder().getInsets(this);
		}
		getLayoutManager().setConstraint(nameLabel,
				new Rectangle(5 - bIns.left, 5 - bIns.top, -1, -1));
		super.validate();
	}

	public void setFireColor(final boolean fireColor) {
		if (fireColor) {
			setColor(SWTResourceManager.getColor(225, 255, 220));
		} else {
			setColor(SWTResourceManager.getColor(255, 170, 170));
		}
	}

	@Override
	public Color getColor() {
		return getForegroundColor();
	}

	@Override
	public void setColor(final Color c) {
		final float hsv[] = c.getRGB().getHSB();
		final float hue = hsv[0];
		final float satuartion = hsv[1];
		float value = hsv[2];
		Color fg;
		if (hue > 180 /* && f[0] < 300 */) {
			fg = SWTResourceManager.getColor(255, 255, 255);
		} else {
			fg = SWTResourceManager.getColor(0, 0, 0);
		}
		nameLabel.setForegroundColor(fg);
		if (highlight) {
			value *= 0.9;
		} else {
			value *= 0.8;
		}

		setForegroundColor(c);
		setBackgroundColor(SWTResourceManager.getColor(new RGB(hue, satuartion,
				value)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	@Override
	public Dimension getMinimumSize(final int wHint, final int hHint) {
		return getPreferredSize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	@Override
	public Dimension getPreferredSize(final int wHint, final int hHint) {
		final int diam = Math.max(nameLabel.getSize().width,
				nameLabel.getSize().height) + 10;
		return new Dimension(diam, 22);
	}

	static class TransitionGhostFigure extends TransitionFigure {

		public TransitionGhostFigure(final TransitionFigure transitionFigure) {
			updateValueDisplay(transitionFigure.nameLabel.getText());
			nameLabel.setForegroundColor(ColorConstants.gray);
			setIcon(transitionFigure.nameLabel.getIcon());
			FigureUtilities.makeGhostShape(this);
		}
	}

	@Override
	public IFigure getGhostFigure() {
		return new TransitionGhostFigure(this);
	}
}