package roneditor.gef.figures.ron;

import java.util.List;

import muvitorkit.gef.editparts.policies.IGhostFigureProvider;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;

import roneditor.gef.figures.objectnet.NodeFigure;
import roneditor.gef.figures.objectnet.Orb;
import ronmodel.HLPlaceType;

public class HLPlaceFigure extends NodeFigure implements IGhostFigureProvider {

	static private final int MINIMUM_WIDTH = 100;

	static private final int RADIUS = 30;

	static private final int MARGIN = 10;

	protected final RectangleFigure contentRectangle;

	protected final ShadedRoundedRectangleFigure backgroundRectangle;

	protected final Orb orb;

	protected final Label typeLabel;

	public HLPlaceFigure() {
		setOutline(false);
		setFill(false);
		setOpaque(false);
		setBackgroundColor(ColorConstants.white);
		setForegroundColor(ColorConstants.black);

		backgroundRectangle = new ShadedRoundedRectangleFigure();
		backgroundRectangle.setOutline(true);
		backgroundRectangle.setOpaque(true);
		backgroundRectangle.setFill(true);
		backgroundRectangle.setForegroundColor(SWTResourceManager.getColor(255,
				255, 255));
		backgroundRectangle.setForegroundColor(SWTResourceManager.getColor(200,
				200, 200));
		add(backgroundRectangle);

		contentRectangle = new RectangleFigure();
		contentRectangle.setOutline(false);
		contentRectangle.setOpaque(false);
		contentRectangle.setFill(false);
		final FlowLayout flowLayout = new FlowLayout();
		flowLayout.setHorizontal(false);
		flowLayout.setMinorSpacing(5);
		flowLayout.setMajorAlignment(FlowLayout.ALIGN_LEFTTOP);
		contentRectangle.setLayoutManager(flowLayout);
		add(contentRectangle);

		nameLabel = new Label(" ");
		nameLabel.setFont(SWTResourceManager.getFont("Sans", 9, SWT.BOLD));
		add(nameLabel);

		orb = new Orb();
		add(orb);

		typeLabel = new Label(" ");
		typeLabel.setFont(SWTResourceManager.getFont("Sans", 14, SWT.BOLD));
		add(typeLabel);

		incomingConnectionAnchor = new ChopboxAnchor(backgroundRectangle);
		outgoingConnectionAnchor = new ChopboxAnchor(backgroundRectangle);

	}

	public void setType(final HLPlaceType type) {
		Color color1, color2;
		if (type.equals(HLPlaceType.NET_LITERAL)) {
			typeLabel.setText("O");
			color1 = SWTResourceManager.getColor(183, 226, 250);
			color2 = SWTResourceManager.getColor(37, 148, 254);
		} else {
			typeLabel.setText("R");
			color1 = SWTResourceManager.getColor(198, 250, 183);
			color2 = SWTResourceManager.getColor(86, 254, 37);
		}
		backgroundRectangle.setForegroundColor(ColorConstants.gray);
		backgroundRectangle.setGradientColor(color1, color2);
		orb.setBackgroundColor(color1);
	}

	private void updateLayout() {

		int w = Math.max(MINIMUM_WIDTH, nameLabel.getPreferredSize().width);
		w = Math.max(w, contentRectangle.getPreferredSize().width);
		w += 2 * MARGIN;

		getLayoutManager().setConstraint(orb,
				new Rectangle((w - RADIUS) / 2, 0, RADIUS, RADIUS));
		getLayoutManager().setConstraint(typeLabel,
				new Rectangle((w - RADIUS) / 2, 0, RADIUS, RADIUS));

		final int y = nameLabel.getPreferredSize().height + 5;

		getLayoutManager().setConstraint(nameLabel,
				new Rectangle(MARGIN, RADIUS, -1, -1));
		getLayoutManager().setConstraint(contentRectangle,
				new Rectangle(MARGIN, RADIUS + y, w, -1));

		final int h = contentRectangle.getPreferredSize().height + y + RADIUS
				- MARGIN;
		getLayoutManager().setConstraint(backgroundRectangle,
				new Rectangle(0, RADIUS - MARGIN, w, h));
	}

	@Override
	public void validate() {
		updateLayout();
		super.validate();
	}

	public RectangleFigure getContentPane() {
		return contentRectangle;
	}

	@Override
	public Dimension getPreferredSize(final int wHint, final int hHint) {
		updateLayout();
		int w = Math.max(MINIMUM_WIDTH, nameLabel.getPreferredSize().width);
		w = Math.max(w, contentRectangle.getPreferredSize().width);
		w += 2 * MARGIN;
		final int h = 2 * (RADIUS - MARGIN)
				+ contentRectangle.getPreferredSize().height
				+ nameLabel.getPreferredSize().height + 5;
		return new Dimension(w, h);
	}

	@Override
	public IFigure getGhostFigure() {
		return new HLPlaceGhostFigure(this);
	}

	static class HLPlaceGhostFigure extends HLPlaceFigure {

		@SuppressWarnings({ "unchecked", "synthetic-access" })
		public HLPlaceGhostFigure(final HLPlaceFigure placeFigure) {

			FigureUtilities.makeGhostShape(orb);
			orb.setForegroundColor(ColorConstants.gray);

			// setType

			nameLabel.setText(placeFigure.nameLabel.getText());
			nameLabel.setForegroundColor(ColorConstants.gray);

			typeLabel.setText(placeFigure.typeLabel.getText());
			typeLabel.setForegroundColor(ColorConstants.gray);

			for (final IFigure childFigure : (List<IFigure>) placeFigure
					.getContentPane().getChildren()) {
				final Label childLabel = (Label) childFigure;
				final Label ghostLabel = new Label(childLabel.getText());
				ghostLabel.setIcon(childLabel.getIcon());
				ghostLabel.setForegroundColor(ColorConstants.gray);
				getContentPane().add(ghostLabel);
			}
		}
	}
}
