package roneditor.gef.figures.ron;

import static java.lang.Math.pow;
import static java.lang.Math.round;
import static java.lang.Math.sqrt;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

public class ShadedRoundedRectangleFigure extends RoundedRectangle {

	private static final int RADIUS = 25;

	private Color gradientColorStart;
	private Color gradientColorEnd;

	public ShadedRoundedRectangleFigure() {
		corner = new Dimension(RADIUS * 2, RADIUS * 2);
	}

	public void setGradientColor(final Color gradientColorStart,
			final Color gradientColorEnd) {
		this.gradientColorStart = gradientColorStart;
		this.gradientColorEnd = gradientColorEnd;
	}

	/**
	 * Fill the shape with a fancy gradient.
	 */
	@Override
	protected void fillShape(final Graphics graphics) {
		final Color oldForegroundColor = getForegroundColor();
		final Color oldBackgroundColor = getForegroundColor();
		if (gradientColorStart != null) {
			graphics.setForegroundColor(gradientColorStart);
		}
		if (gradientColorEnd != null) {
			graphics.setBackgroundColor(gradientColorEnd);
		}

		if (graphics.getXORMode()) {
			graphics.fillRoundRectangle(getBounds(), corner.width,
					corner.height);
		} else {
			// graphics.fillRoundRectangle(getBounds(), corner.width,
			// corner.height);

			// this just doesn't work so well with zoom level > 100%

			// fill rectangle in center of the rounded rectangle with gradient
			final Rectangle rect = getBounds().getCopy();
			rect.x += RADIUS;
			rect.width -= 2 * RADIUS;
			graphics.fillGradient(rect, true);

			// fill remaining segments with lines
			rect.width = 1;
			for (int i = 1; i <= RADIUS; i++) {
				// calculate length of stripe
				final int circleHeight = (int) round(sqrt(pow(RADIUS, 2)
						- pow(i, 2)));
				rect.height = circleHeight * 2 + getBounds().height - 2
						* RADIUS;

				// fill left segment with a line
				rect.x = getBounds().x + RADIUS - i;
				rect.y = getBounds().y + (getBounds().height - rect.height) / 2;
				graphics.fillGradient(rect, true);

				// fill right segment with a line
				rect.x = getBounds().x + getBounds().width - RADIUS + i - 1;
				rect.y = getBounds().y + (getBounds().height - rect.height) / 2;
				graphics.fillGradient(rect, true);
			}

		}
		graphics.setForegroundColor(oldForegroundColor);
		graphics.setBackgroundColor(oldBackgroundColor);
	}

	@Override
	protected void outlineShape(final Graphics graphics) {
		super.outlineShape(graphics);
	}

}
