package roneditor.model.properties;

import java.util.ArrayList;

import muvitorkit.properties.AbstractPropertySource;
import muvitorkit.properties.LocationPropertySource;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.HLPlace;

/**
 * Property source for a HLPlace.
 */
public class HLPlacePropertySource extends AbstractPropertySource<HLPlace> {

	private static enum ID {
		NAME, TYPE, LOCATION
	}

	public HLPlacePropertySource(final HLPlace place) {
		super(place);
	}

	@Override
	protected IPropertyDescriptor[] createPropertyDescriptors() {
		final ArrayList<IPropertyDescriptor> descriptorList = new ArrayList<IPropertyDescriptor>();
		descriptorList.add(new TextPropertyDescriptor(ID.NAME, "Name"));
		final String[] types = { "NET", "RULE" };
		descriptorList.add(new ComboBoxPropertyDescriptor(ID.TYPE, "Type",
				types));
		descriptorList.add(new TextPropertyDescriptor(ID.LOCATION, "Location"));
		return descriptorList.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java
	 * .lang.Object)
	 */
	@Override
	public Object getPropertyValue(final Object id) {
		if (id instanceof ID) {
			switch ((ID) id) {
			case NAME:
				return getModel().getName();
			case TYPE:
				return Integer.valueOf(getModel().getType().getValue());
			case LOCATION:
				return new LocationPropertySource(getModel().getX(), getModel()
						.getY());
			}
		}
		return null;
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java
	 * .lang.Object, java.lang.Object)
	 */
	@Override
	public void setPropertyValue(final Object id, final Object value) {
		switch ((ID) id) {
		case NAME:
			getModel().setName((String) value);
			break;
		case LOCATION:
			final Point newLoc = (Point) value;
			getModel().setX(newLoc.x);
			getModel().setY(newLoc.y);
			break;
		default:
		}
	}
}
