package roneditor.model.properties;

import java.util.ArrayList;

import muvitorkit.properties.AbstractPropertySource;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.ONArc;

/**
 * Property source for a Place.
 */
public class ONArcPropertySource extends AbstractPropertySource<ONArc> {

	private static enum ID {
		NAME, WEIGHT
	}

	public ONArcPropertySource(final ONArc onArc) {
		super(onArc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.properties.AbstractPropertySource#createPropertyDescriptors()
	 */
	@Override
	protected IPropertyDescriptor[] createPropertyDescriptors() {
		final ArrayList<IPropertyDescriptor> descriptorList = new ArrayList<IPropertyDescriptor>();
		descriptorList.add(new TextPropertyDescriptor(ID.NAME, "Name"));
		descriptorList.add(new TextPropertyDescriptor(ID.WEIGHT, "Weight"));
		return descriptorList.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java
	 * .lang.Object)
	 */
	@Override
	public Object getPropertyValue(final Object id) {
		if (id instanceof ID) {
			switch ((ID) id) {
			case NAME:
				return getModel().getName();
			case WEIGHT:
				return String.valueOf(getModel().getWeight());
			}
		}
		return null;
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java
	 * .lang.Object, java.lang.Object)
	 */
	@Override
	public void setPropertyValue(final Object id, final Object value) {
		switch ((ID) id) {
		case NAME:
			getModel().setName((String) value);
			break;
		case WEIGHT:
			try {
				final int i = Integer.parseInt((String) value);
				if (i > 0) {
					getModel().setWeight(i);
				}
			} catch (final NumberFormatException e) {
			}
			break;
		default:
		}
	}

}
