package roneditor.model.properties;

import java.util.ArrayList;

import muvitorkit.properties.AbstractPropertySource;
import muvitorkit.properties.LocationPropertySource;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.Mapping;
import ronmodel.ONPlace;

/**
 * Property source for a ONPlace.
 */
public class ONPlacePropertySource extends AbstractPropertySource<ONPlace> {

	private static enum ID {
		NAME, TOKENS, CAPACITY, LOCATION, MAPPINGS
	}

	public ONPlacePropertySource(final ONPlace place) {
		super(place);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.properties.AbstractPropertySource#createPropertyDescriptors()
	 */
	@Override
	protected IPropertyDescriptor[] createPropertyDescriptors() {
		final ArrayList<IPropertyDescriptor> descriptorList = new ArrayList<IPropertyDescriptor>();
		descriptorList.add(new TextPropertyDescriptor(ID.NAME, "Name"));
		descriptorList.add(new TextPropertyDescriptor(ID.TOKENS, "Tokens"));
		descriptorList.add(new TextPropertyDescriptor(ID.CAPACITY, "Capacity"));
		descriptorList.add(new PropertyDescriptor(ID.LOCATION, "Location"));
		descriptorList.add(new PropertyDescriptor(ID.MAPPINGS, "Mappings"));
		return descriptorList.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java
	 * .lang.Object)
	 */
	@Override
	public Object getPropertyValue(final Object id) {
		if (id instanceof ID) {
			switch ((ID) id) {
			case NAME:
				return getModel().getName();
			case TOKENS:
				return Integer.toString(getModel().getTokens());
			case CAPACITY:
				return Integer.toString(getModel().getCapacity());
			case LOCATION:
				return new LocationPropertySource(getModel().getX(), getModel()
						.getY());
			case MAPPINGS:
				final StringBuilder sb = new StringBuilder();
				for (final Mapping mapping : getModel().getIsSourceInMappings()) {
					sb.append("(->");
					sb.append(mapping.getTarget().getName());
					sb.append("@");
					sb.append(mapping.getTarget().getObjectNet().getName());
					sb.append(")");
				}
				for (final Mapping mapping : getModel().getIsTargetInMappings()) {
					sb.append("(");
					sb.append(mapping.getSource().getName());
					sb.append("@");
					sb.append(mapping.getSource().getObjectNet().getName());
					sb.append("->)");
				}
				return sb.toString();
			}
		}
		return null;
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java
	 * .lang.Object, java.lang.Object)
	 */
	@Override
	public void setPropertyValue(final Object id, final Object value) {
		switch ((ID) id) {
		case NAME:
			getModel().setName((String) value);
			break;
		case TOKENS:
			try {
				final int i = Integer.parseInt((String) value);
				if (i >= 0
						&& (getModel().getCapacity() >= i || getModel()
								.getCapacity() == 0)) {
					getModel().setTokens(i);
				}
			} catch (final NumberFormatException e) {
			}
			break;
		case CAPACITY:
			try {
				final int i = Integer.parseInt((String) value);
				if (i >= 0) {
					if (getModel().getTokens() > i && i != 0) {
						getModel().setTokens(i);
					}
					getModel().setCapacity(i);
				}
			} catch (final NumberFormatException e) {
			}
			break;
		case LOCATION:
			final Point newLoc = (Point) value;
			getModel().setX(newLoc.x);
			getModel().setY(newLoc.y);
			break;
		default:
		}
	}
}
