package roneditor.model.properties;

import java.util.ArrayList;

import muvitorkit.properties.AbstractPropertySource;
import muvitorkit.properties.LocationPropertySource;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ronmodel.Mapping;
import ronmodel.ONTransition;

/**
 * Property source for a ONTransition.
 */
public class ONTransitionPropertySource extends
		AbstractPropertySource<ONTransition> {

	private static enum ID {
		NAME, LOCATION, MAPPINGS
	}

	public ONTransitionPropertySource(final ONTransition onTransition) {
		super(onTransition);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.properties.AbstractPropertySource#createPropertyDescriptors()
	 */
	@Override
	protected IPropertyDescriptor[] createPropertyDescriptors() {
		final ArrayList<IPropertyDescriptor> descriptorList = new ArrayList<IPropertyDescriptor>();
		descriptorList.add(new TextPropertyDescriptor(ID.NAME, "Name"));
		descriptorList.add(new TextPropertyDescriptor(ID.LOCATION, "Location"));
		descriptorList.add(new PropertyDescriptor(ID.MAPPINGS, "Mappings"));
		return descriptorList.toArray(new IPropertyDescriptor[] {});
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java
	 * .lang.Object)
	 */
	@Override
	public Object getPropertyValue(final Object id) {
		if (id instanceof ID) {
			switch ((ID) id) {
			case NAME:
				return getModel().getName();
			case LOCATION:
				return new LocationPropertySource(getModel().getX(), getModel()
						.getY());
			case MAPPINGS:
				final StringBuilder sb = new StringBuilder();
				for (final Mapping mapping : getModel().getIsSourceInMappings()) {
					sb.append("(->");
					sb.append(mapping.getTarget().getName());
					sb.append("@");
					sb.append(mapping.getTarget().getObjectNet().getName());
					sb.append(")");
				}
				for (final Mapping mapping : getModel().getIsTargetInMappings()) {
					sb.append("(");
					sb.append(mapping.getSource().getName());
					sb.append("@");
					sb.append(mapping.getSource().getObjectNet().getName());
					sb.append("->)");
				}
				return sb.toString();
			}
		}
		return null;
	}

	/*
	 * @see
	 * org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java
	 * .lang.Object, java.lang.Object)
	 */
	@Override
	public void setPropertyValue(final Object id, final Object value) {
		switch ((ID) id) {
		case NAME:
			getModel().setName((String) value);
			break;
		case LOCATION:
			final Point newLoc = (Point) value;
			getModel().setX(newLoc.x);
			getModel().setY(newLoc.y);
			break;
		default:
		}
	}

}
