package roneditor.ui.objectnet;

import muvitorkit.animation.demo.AnimatingDemoAction3;
import muvitorkit.gef.palette.MuvitorPaletteRoot;
import muvitorkit.ui.ContextMenuProviderWithActionRegistry;
import muvitorkit.ui.MuvitorPage;
import muvitorkit.ui.MuvitorPageBookView;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.objectnet.FireONTransitionAction;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import aggSim.analyzer.LocalPlaceConflictAnalysisAction;

/**
 * <p>
 * This is an implementation of {@link MuvitorPageBookView} to display
 * {@link ObjectNet}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetEditPartFactory} and
 * {@link ObjectNetPageContextMenuProvider}, configures the keyhandler and
 * creates actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 * 
 * @author Tony Modica
 */
public class ObjectNetPage extends MuvitorPage {

	public ObjectNetPage(final MuvitorPageBookView view) {
		super(view);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ui.part.AbstractMultiViewerPage#createCustomActions()
	 */
	@Override
	protected void createCustomActions() {
		registerSharedActionAsHandler(ActionFactory.COPY.getId());
		registerSharedActionAsHandler(ActionFactory.CUT.getId());
		registerSharedActionAsHandler(ActionFactory.PASTE.getId());
		registerAction(new LocalPlaceConflictAnalysisAction(getEditor()));
		registerActionAsHandler(new FireONTransitionAction(getEditor()));
		registerAction(new AnimatingDemoAction3(getEditor()));
		// registerSharedAction(ExportPNMLAction.ID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler
	 * (org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory
	 * ()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider
	 * ()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new ObjectNetPageContextMenuProvider(viewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected MuvitorPaletteRoot createPaletteRoot() {
		return new ObjectNetPaletteRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		return new EObject[] { getModel() };
	}
}
