package roneditor.ui.wizards;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;

public class CreateTokenWizardPage extends WizardPage implements Listener {

	private Text tokenName;

	protected CreateTokenWizardPage(final IWorkbench workbench,
			final IStructuredSelection selection) {
		super("Create a new Token");
		setTitle("Create a new Token");
		setDescription("Create a new Token on HL-Places");
	}

	@Override
	public void createControl(final Composite parent) {
		GridData gd;
		final Composite composite = new Composite(parent, SWT.NULL);

		final GridLayout gl = new GridLayout();
		final int ncol = 3;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		// Name Text Field
		new Label(composite, SWT.NONE).setText("Name:");
		tokenName = new Text(composite, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol - 1;
		tokenName.setLayoutData(gd);
		tokenName.setFocus();
		setControl(composite);
		addListeners();
	}

	@Override
	public boolean isPageComplete() {
		final CreateTokenWizard wizard = (CreateTokenWizard) getWizard();
		if (tokenName.getText().isEmpty()) {
			return false;
		}
		wizard.setTokenName(tokenName.getText());
		return true;
	}

	private void addListeners() {
		tokenName.addListener(SWT.KeyUp, this);
	}

	@Override
	public void handleEvent(final Event event) {
		if (tokenName.getText().isEmpty()) {
			setErrorMessage("Name of the Token is empty");
		} else {
			setErrorMessage(null);
		}
		getWizard().getContainer().updateButtons();
	}

}
